using System;
using System.Data;
using System.Data.Common;		// Contains DataTableMapping
using System.Collections;
using System.ComponentModel;
using iAnywhere.Data.UltraLite;

namespace iAnywhere.Data.UltraLite.CustomDataSet
{
	/// <summary>
	/// Summary description for CustomDataSet.
	/// Does not have much of the functionality of a System.Data.DataSet
	/// Note: You cannot set a DataGrid's DataSource to a CustomDataSet.
	/// You must set the DataGrid's DataSource to a CustomDataTable.
	/// </summary>
	public class CustomDataSet : IListSource
	{
		private Hashtable					_tables;
		private DataTableMappingCollection	_tableMappings;
		private MissingMappingAction		_missingMappingAction;

		public CustomDataSet()
		{
			_tables = new Hashtable();
			_missingMappingAction = MissingMappingAction.Passthrough;
			_tableMappings = new DataTableMappingCollection();
		}
		public bool ContainsListCollection {
			get {
				return false;
			}
		}

		public IList GetList() {
			//return _tables;
			return null;  //to compile
		}

		public DataTableMappingCollection TableMappings {
			get {
				return _tableMappings;
			}
		}
		public MissingMappingAction MissingMappingAction {
			get {
				return _missingMappingAction;
			}
			set {
				_missingMappingAction = value;
			}
		}

		/* Fills a new CustomDataTable with the contents of the reader.
		 * If srcTable has an alias, then use its alias as a key in the
		 * hash table of CustomDataTable objects.  Otherwise, use
		 * srcTable as the key.
		 * 
		 * Also, this class does not check for collisions of table names.
		 * For example, you could make a table's alias the same as a 
		 * another table's name.  When you call Fill on either table,
		 * the current entry in the hash table is simply removed.  So,
		 * running getTable on a name for which two tables exist will
		 * return the most recently-filled table.
		 */
		public void Fill( ULDataReader reader )
		{
			DataTableMapping tm = null;
			string srcTable = reader.Schema.Name;
			string dataTableName = reader.Schema.Name;
			if (_tableMappings.Contains( srcTable )) {
				tm = _tableMappings[ srcTable ];
				dataTableName = tm.DataSetTable;
			}
			_tables.Remove( dataTableName );
			CustomDataTable t = new CustomDataTable();
			// Pass the CustomDataSet's MissingMappingAction
			// setting to its new table.
			t.MissingMappingAction = _missingMappingAction;  
			t.Fill( reader, tm );				// Fill takes care of column mappings			
			_tables.Add( dataTableName, t );	// Add the table to the hash table
		}

		/* Gets the table with the specified name.  If tableName is not found,
		 * and it has an alias, then check if a table exists that has the
		 * alias name.
		 * Returns null if table cannot be found.
		 */ 
		public CustomDataTable getTable( string tableName ) 
		{
			DataTableMapping tm = null;
			if (_tables.ContainsKey( tableName )) {
				return (CustomDataTable) _tables[ tableName ];
			} else {
				if (_tableMappings.Contains( tableName )) {
					tm = _tableMappings[ tableName ];
					if (_tables.ContainsKey( tm.DataSetTable )) {
						return (CustomDataTable) _tables[ tm.DataSetTable ];
					}
				}
			}
			return null;  
		}
	}
}
