using System;
using System.ComponentModel;

namespace iAnywhere.Data.UltraLite.CustomDataSet {

    /// <summary><para>This class allows the DataGrid to query the type and the value of the table
    /// column that the class is associated with. When the class is constructed, it is given the
    /// column number and type of the column that it should support. The GetValue() method allows
    /// the DataGrid to retrieve the actual data value of the column, given a particular row.
    /// </para></summary>
    internal class CustomTableColumn : PropertyDescriptor
    {
	private String 	_name; // name of the column - not used at the moment
	private int 	_columnNumber; // the column number within the table
	private Type 	_type; // the data type of the column

	// constructor
	internal CustomTableColumn(int number, String name, Type type )  : base( name, null ) {
	    _columnNumber = number;
	    _name = name;
	    _type = type;
	}

	public int ColNumInDB {
		get {
			return _columnNumber;
		}
	}

	// retrieves the type of the row
	public override Type ComponentType {
		get {
			return typeof( CustomTableRow );
		}
	}

	// retrieves the type of the column
	public override Type PropertyType {
	    get {
		return _type;
	    }
	}

	// gets the data value for the given row
	public override object GetValue( object component ) {

	    CustomTableRow pd = (CustomTableRow) component;
	    if( pd[_columnNumber] == null ) {
			return DBNull.Value;
	    }
	    return pd[_columnNumber];
	}

	// indicates that the property is read-only
	public override bool IsReadOnly {
	    get {
		return true;
	    }
	}

	// indicates that resetting the object might change the value
	public override bool CanResetValue( object component) {
	    return false;
	}

	// returns false, since this class is not meant for serialization at the moment
	public override bool ShouldSerializeValue( object component ) {
	    return false;
	}

	// Unsupported methods

	public override void SetValue( object component, object value ) {
	    throw new NotSupportedException();
	}

	public override void ResetValue( object component ) {
	    throw new NotSupportedException();
	}
    }
}
