using System;
using System.Collections;
using System.ComponentModel;
using iAnywhere.Data.UltraLite;

namespace iAnywhere.Data.UltraLite.CustomDataSet {

    /// <summary><para>This classes contains type converstion
    /// related methods.</para></summary>
    internal class ULDotNETBridge
    {
	// Returns the .NET equivalent of the ULDbType 
	internal static System.Type GetNETType(ULDbType ulType) 
	{
		
	    switch(ulType)
	    {
		    case ULDbType.Binary: 
			    return typeof(System.Byte[]);
		    case ULDbType.Bit:
			    return typeof(bool);					
		    case ULDbType.Char:
			case ULDbType.VarChar:
			    return typeof(System.String);
		    case ULDbType.Date:
			    return typeof(System.DateTime);
		    case ULDbType.Double:
			    return typeof(System.Double);
		    case ULDbType.LongBinary:
			    return typeof(System.Byte[]);
		    case ULDbType.LongVarchar:
			    return typeof(System.String);
		    case ULDbType.Decimal:
			    return typeof(System.String);
		    case ULDbType.Real:
			    return typeof(System.Single);
		    case ULDbType.BigInt:
			    return typeof(System.Int64);
		    case ULDbType.Integer:
			    return typeof(System.Int32);
		    case ULDbType.SmallInt:
			    return typeof(System.Int16);
		    case ULDbType.Time:
			    return typeof(System.TimeSpan);
		    case ULDbType.DateTime:
			    return typeof(System.DateTime);
		    case ULDbType.TinyInt:
			    return typeof(System.Byte);
		    case ULDbType.UnsignedBigInt:
			    return typeof(System.UInt64);
		    case ULDbType.UnsignedInt:
			    return typeof(System.UInt32);
		    case ULDbType.UnsignedSmallInt:
			    return typeof(System.UInt16);
			case ULDbType.UniqueIdentifier:
				return typeof(System.Guid);
		    default:
			    return null;
	    }
	}

	/* Populates the blank row with the current row of the table
	 */
	internal static bool GetCurrentRowData( 
				ULDataReader table, 
				ULCursorSchema schema, 
				CustomTableRow row ) {
	    for( int j = 0; j < schema.ColumnCount; j++ ) {
			if( !getRowColData( table, schema, row, j ) ) return false;
	    }
	    return true;
	}

	/* Populates the specified column of the blank row 
	 * using the table's current row 
	 */
	private static bool getRowColData( 
				ULDataReader table, 
				ULCursorSchema schema, 
				CustomTableRow row, 
				int colID )
	{
	    //call appropriate method for retrieving column data in
	    //row depending on type

	    if( table.IsDBNull(colID) ) {
			row[colID] = null;
			return true;
	    }

	    ULDbType colType = schema.GetColumnULDbType(colID);
	    
	    switch( colType )
	    {
		case ULDbType.Binary: 					
			row[colID] = table.GetBytes(colID);
			break;
		case ULDbType.Bit:
			row[colID] = table.GetBoolean(colID);
			break;
		case ULDbType.Char:					
		case ULDbType.VarChar:
			row[colID] = table.GetString(colID);
			break;					
		case ULDbType.Date:
			row[colID] = table.GetDateTime(colID);
			break;					
		case ULDbType.Double:
			row[colID] = table.GetDouble(colID);
			break;										
		case ULDbType.LongBinary:
			row[colID] = table.GetBytes(colID);
			break;										
		case ULDbType.LongVarchar:
			row[colID] = table.GetString(colID);
			break;															
		case ULDbType.Numeric:
			row[colID] = table.GetString(colID);
			break;															
		case ULDbType.Real:
			row[colID] = table.GetFloat(colID);
			break;
		case ULDbType.BigInt:
			row[colID] = table.GetInt64(colID);
			break;					
		case ULDbType.Integer:
			row[colID] = table.GetInt32(colID);
			break;
		case ULDbType.SmallInt:
			row[colID] = table.GetInt16(colID);
			break;
		case ULDbType.Time:
			row[colID] = table.GetTimeSpan(colID);
			break;
		case ULDbType.TimeStamp:
			row[colID] = table.GetDateTime(colID);
			break;
		case ULDbType.TinyInt:
			row[colID] = table.GetBytes(colID);
			break;					
		case ULDbType.UnsignedBigInt:
			row[colID] = table.GetUInt64(colID);
			break;
		case ULDbType.UnsignedInt:
			row[colID] = table.GetUInt32(colID);
			break;
		case ULDbType.UnsignedSmallInt:
			row[colID] = table.GetUInt16(colID);
			break;
		case ULDbType.UniqueIdentifier:
			row[colID] = table.GetGuid(colID);
			break;
		default:
			return false;				
	    }

	    return true;
	}
    }
}   
