/* Note: In the Solution Explorer, make sure that the 
 * path to the ulnet9.dll is correct.  If it is not,
 * then remove it, and add a correct link.
 */

using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Data;
using System.Data.Common;
using iAnywhere.Data.UltraLite;
using iAnywhere.Data.UltraLite.CustomDataSet;

namespace DataGridLargeTables
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class frmDGLargeTables : System.Windows.Forms.Form
	{
		private System.Windows.Forms.DataGrid dataGrid1;
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.ComboBox cboTableNames;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.TextBox txtReaderName;
		private System.Windows.Forms.TextBox txtDataName;
		private ULConnection _conn;
		private CustomDataSet _dataSet;
		private int prevSelectedTableIndex;

		public frmDGLargeTables()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			
			prevSelectedTableIndex = -1;
			_dataSet = new CustomDataSet();

			// This MissingMappingAction setting indicates that when 
			// a mapping is missing, just use the name from the reader.
			// There are other settings available.
			_dataSet.MissingMappingAction = MissingMappingAction.Passthrough;			
		}
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.cboTableNames = new System.Windows.Forms.ComboBox();
			this.dataGrid1 = new System.Windows.Forms.DataGrid();
			this.label1 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.txtReaderName = new System.Windows.Forms.TextBox();
			this.txtDataName = new System.Windows.Forms.TextBox();
			// 
			// cboTableNames
			// 
			this.cboTableNames.Location = new System.Drawing.Point(8, 8);
			this.cboTableNames.Size = new System.Drawing.Size(224, 22);
			this.cboTableNames.SelectedIndexChanged += new System.EventHandler(this.cboTableNames_SelectedIndexChanged_1);
			// 
			// dataGrid1
			// 
			this.dataGrid1.Location = new System.Drawing.Point(8, 96);
			this.dataGrid1.Size = new System.Drawing.Size(224, 168);
			this.dataGrid1.Text = "dataGrid1";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 40);
			this.label1.Size = new System.Drawing.Size(96, 20);
			this.label1.Text = "Reader Name:";
			// 
			// label3
			// 
			this.label3.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Regular);
			this.label3.Location = new System.Drawing.Point(8, 64);
			this.label3.Size = new System.Drawing.Size(104, 20);
			this.label3.Text = "DataTable Name:";
			// 
			// txtReaderName
			// 
			this.txtReaderName.Location = new System.Drawing.Point(104, 40);
			this.txtReaderName.Size = new System.Drawing.Size(128, 22);
			this.txtReaderName.Text = "";
			// 
			// txtDataName
			// 
			this.txtDataName.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Regular);
			this.txtDataName.Location = new System.Drawing.Point(104, 64);
			this.txtDataName.Size = new System.Drawing.Size(128, 22);
			this.txtDataName.Text = "";
			// 
			// frmDGLargeTables
			// 
			this.Controls.Add(this.txtDataName);
			this.Controls.Add(this.txtReaderName);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.dataGrid1);
			this.Controls.Add(this.cboTableNames);
			this.Menu = this.mainMenu1;
			this.Text = "DataGrid Large Tables";
			this.Load += new System.EventHandler(this.frmDGLargeTables_Load);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>

		static void Main() 
		{
			Application.Run(new frmDGLargeTables());
		}
		/*
			Opens connection to the database
		*/
		private bool InitDB() 
		{		

			try 
			{
				_conn = new ULConnection(
					"UID=DBA;PWD=SQL;DBF=\\Program Files\\DataGridLargeTables\\ul_custapi.udb");
				_conn.Open();
			}
			catch ( System.Exception t ) 
			{
				MessageBox.Show( "Exception: " + t.Message);
				return false;
			}
			return true;
		}
		/*
			Populate comboBox with Table Names using the Schema information
		*/
		private bool PopulateComboBox() 
		{
			int numTables;
			try 
			{
				if( !_conn.Schema.IsOpen ) 
				{
					System.Windows.Forms.MessageBox.Show( "Connection Schema is not Open, cannot populate ComboBox.", "Error");
					return false;
				}
				numTables = _conn.Schema.TableCount;			
				for( int i=1; i<=numTables; i++ ) 
				{
					cboTableNames.Items.Add(_conn.Schema.GetTableName(i));
				}				
			}
			catch ( System.Exception t ) 
			{
				MessageBox.Show( "Exception: " + t.Message);
				return false;
			}
			return true;
		}

		/* This method shows two different ways of creating name maps.
		 * A name map allows you to display more user-friendly table 
		 * or column names in the data grid.
		 * The first way requires less code, but does not allow you
		 * to change column widths.
		 * The second way requires a lot more code, but does allow you
		 * to change column widths.  A disadvantage of the second way
		 * is that you have to make a DataGridTextBoxColumn for every column
		 * that you want to show.
		 * 
		 * The two ways can be used to complement each other, but note that
		 * DataGridTableStyle objects operate after the mappings in
		 * ULDataAdapter.  For example, if a mapping in the 
		 * ULDataAdapter.TableMappings
		 * adds an alias of 'Customer' to the 'ULCustomer' table, then
		 * the DataGridTableStyle object should refer to the 'Customer' table.
		 * 
		 * There is the third way of column mapping through aliases in SQL code
		 * To accomplish the same mapping as in the first example, the SQL
		 * select statement would be 
		 * 'SELECT cust_id, cust_name AS CustName FROM ULCustomer'
		 */
		private void createTableStyles() 
		{
			/***** First way *******/
			// Make a map for the table with a name that is more user-friendly.
			// The reader's name is its SQL statement.
			// Mappings work as Add(<actual_name>, <alias_name>)
			DataTableMapping tm = _dataSet.TableMappings.Add(
				"SELECT * FROM ULCustomer", "Customer");

			// Make column mappings.
			// We are purposely missing a mapping for ULCustomer.cust_id
			// to demonstrate the currently selected MissingMappingAction of Passthrough.
			// The cust_id column name will pass through to the CustomDataTable.
			// By uncommenting the following line, the cust_id column will
			// have the name 'CustID'.

			//tm.ColumnMappings.Add("ULCustomer.cust_id", "CustID");
			tm.ColumnMappings.Add("ULCustomer.cust_name", "CustName");
			/****** End of first way *******/

			/******* Second way ********/
			DataGridTableStyle tsULProduct = new DataGridTableStyle();
			tsULProduct.MappingName = "SELECT * FROM ULProduct";			

			DataGridColumnStyle prodIdCol2 = new DataGridTextBoxColumn();
			prodIdCol2.MappingName = "ULProduct.prod_id";
			prodIdCol2.HeaderText = "ProductID";
			prodIdCol2.Width = 40;			
			
			DataGridColumnStyle priceCol = new DataGridTextBoxColumn();
			priceCol.MappingName = "ULProduct.price";
			priceCol.HeaderText = "Price";
			priceCol.Width = 30;			

			DataGridColumnStyle prodNameCol = new DataGridTextBoxColumn();
			prodNameCol.MappingName = "ULProduct.prod_name";
			prodNameCol.HeaderText = "Product";
			prodNameCol.Width = 150;
			
			//add ColumnStyles to TableStyle
			
			tsULProduct.GridColumnStyles.Add(prodIdCol2);
			tsULProduct.GridColumnStyles.Add(priceCol);
			tsULProduct.GridColumnStyles.Add(prodNameCol);

			dataGrid1.TableStyles.Add(tsULProduct);
			/******** End of second way **********/
		}

		private void frmDGLargeTables_Load(object sender, System.EventArgs e)
		{
			if( InitDB() && PopulateComboBox() ) 
			{
				createTableStyles();
				cboTableNames.SelectedIndex = 0;
			}
		}

		private void cboTableNames_SelectedIndexChanged_1(object sender, System.EventArgs e)
		{
			ULCommand command;
			ULDataReader reader;

			if( cboTableNames.SelectedIndex != prevSelectedTableIndex ) 
			{
				// Update the currently selected table
				prevSelectedTableIndex = cboTableNames.SelectedIndex;

				// Create the SQL Select command
				command = _conn.CreateCommand();
				command.CommandText = "SELECT * FROM " + cboTableNames.Text;

				// Execute the SQL Select command
				reader = command.ExecuteReader(); 				

				// _dataSet is basically a wrapper for ULDataReader that
				// enables the DataGrid control to view its contents.
				// In this Fill call, table name mappings are handled.
				_dataSet.Fill( reader );  
				
				// Get the table using its UDB name
				// getTable also works if given the table's alias name
				CustomDataTable t = _dataSet.getTable( 
					"SELECT * FROM " + cboTableNames.Text );  

				// Note: You cannot set a DataGrid's DataSource to a CustomDataSet
				// because the Compact .NET Framework does not support setting
				// DataGrid.DataMember
				// As a result, you must set the DataGrid's DataSource 
				// to a CustomDataTable.

				if (t!=null) 
				{
					dataGrid1.DataSource = t;	// Set the DataSource
					txtReaderName.Text = reader.Schema.Name;
					txtDataName.Text = t.Name;
				} else {
					MessageBox.Show("Table not found.");
				}

				/* To skip the CustomDataSet object, and just use a CustomDataTable,
				 * comment the above 10 lines (up to and including '_dataSet.Fill ...') 
				 * and uncomment the following 6 lines (up to and including
				 * 'txtDataName.Text = ...'.  
				 * 
				 * To make table mappings with the following code, you can make a 
				 * DataTableMapping object, and pass it to the overloaded t.Fill method.
				 * Alternatively, in createTableStyles, the second way of name
				 * mapping will work with the following code.  
				 */
				 
//				CustomDataTable t = new CustomDataTable();
//				t.Fill( reader );
//				dataGrid1.DataSource = t;
//				txtReaderName.Text = reader.Schema.Name;
//				txtDataName.Text = t.Name;
			}
		}
	}
}
